import java.applet.Applet;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.text.DecimalFormat;
import java.util.HashSet;
import java.util.Set;


public class Schema {
	
	private Field[][] fields;
	private double ratioX;
	private double ratioY;
	private Set<Objekt> objects;
	
	public static Point2D.Double getVector(double length, double angle){
		return new Point2D.Double(length*Math.cos(angle-Math.PI/2),length*Math.cos(angle));
	}
	
	/**
	 * Prida objekt s gravitaciou do schemy
	 * @param object
	 * @throws Exception
	 */
	public void addObject(Objekt object) throws Exception{
		objects.add(object);
		fields[convertToSchemaY(object.getY())][convertToSchemaX(object.getX())].setObject(object);
		update();
	}
	
	/**
	 * Vytvori schemu
	 * @param resolutionX horizontalne rozlisenie schemy
	 * @param resolutionY vertikalne rozlisenie schemy
	 * @param worldWidht sirka sveta (obrazku)
	 * @param worldHeight vyska sveta (obrazku)
	 */
	public Schema(int resolutionX, int resolutionY, int worldWidht, int worldHeight) {
		objects = new HashSet<Objekt>();
		ratioX = worldWidht / resolutionX;
		ratioY = worldHeight / resolutionY;
		fields = new Field[resolutionY][resolutionX];
		for (int i = 0; i < fields.length; i++) {
			for (int j = 0; j < fields[0].length; j++) {
				fields[i][j] = new Field(convertToWorldX(j), convertToWorldY(i));
			}
		}
	}
	
	/**
	 * Aktualizuje vektory sil na kazdom policku
	 */
	private void update(){
		for (int i = 0; i < fields.length; i++) {
			for (int j = 0; j < fields[0].length; j++) {
				double fieldX = convertToWorldX(j);
				double fieldY = convertToWorldY(i);
				double angle=0;
				double force=0;
				
				Point2D.Double finalVector = getVector(force, angle);
				for (Objekt obstacle : objects) {
					double distanceObst = Point2D.distance(fieldX, fieldY, obstacle.getX(), obstacle.getY());
					double forceObst = (distanceObst>obstacle.getGravityRadius())?
							0:obstacle.getGravity() - obstacle.getGravity()*(distanceObst/obstacle.getGravityRadius());
					double angleObst = Math.atan2((obstacle.getX()-fieldX),(-(obstacle.getY()-fieldY)));
					Point2D.Double vecObst = getVector(forceObst, angleObst);
					finalVector.setLocation(finalVector.getX()+vecObst.getX(), finalVector.getY()+vecObst.getY());
				}
				fields[i][j].setAngle(Math.atan2(finalVector.getX(), finalVector.getY()));
				fields[i][j].setForce(finalVector.distance(0, 0));
			}
		}
	}
	
	public int convertToSchemaX(double x){
		return (int)Math.round(x/ratioX);
	}

	public int convertToSchemaY(double y){
		return (int)Math.round(y/ratioY);
	}
	
	public double convertToWorldX(int x){
		return ratioX*x;
	}
	
	public double convertToWorldY(int y){
		return ratioY*y;
	}
	
	public void draw(Graphics g){
		int size = 6;
		int padding = 30;
		for (int i = 0; i < fields.length; i++) {
			for (int j = 0; j < fields[0].length; j++) {
				Point2D.Double vector = fields[i][j].getVector();
				if (fields[i][j].hasObject()){
					if (fields[i][j].getObject().getGravity()>=0){
						g.setColor(Color.red);
					}else{
						g.setColor(Color.blue);
					}
					g.fillOval(j*padding, i*padding, size, size);
					continue;
				}
				int x1 = j*padding;
				int y1 = i*padding;
				int xC = x1+size/2;
				int yC = y1+size/2;
				
				g.setColor(Color.black);
				g.drawOval(x1, y1, size, size);
				
				g.drawLine(xC,yC,(int)(xC+vector.getX()),(int)(yC-vector.getY()));
				g.setColor(Color.red);
				g.drawLine((int)(xC+vector.getX()),(int)(yC-vector.getY()), (int)(xC+vector.getX()),(int)(yC-vector.getY()));
			}
		}
	}
	
	public String toStringAngle(){
		StringBuffer result = new StringBuffer();
		DecimalFormat df = new DecimalFormat("000.00");
		df.setPositivePrefix("+");
		for (int i = 0; i < fields.length; i++) {
			for (int j = 0; j < fields[0].length; j++) {
				if (fields[i][j].hasObject()){
					result.append(fields[i][j].getObject()+"|");
					continue;
				}
				result.append(df.format(fields[i][j].getDegrees())+"|");
			}
			result.append("\n");
		}
		return result.toString();
	}
	
	public String toStringForce(){
		StringBuffer result = new StringBuffer();
		DecimalFormat df = new DecimalFormat("0000.00");
		for (int i = 0; i < fields.length; i++) {
			for (int j = 0; j < fields[0].length; j++) {
				if (fields[i][j].hasObject()){
					result.append(fields[i][j].getObject()+"|");
					continue;
				}
				result.append(df.format(fields[i][j].getForce())+"|");
			}
			result.append("\n");
		}
		return result.toString();
	}
	
	
	/**
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {
		/*System.out.println("======= test gravitacie");
		Schema s = new Schema(20, 20, 100, 100);
		s.setDestination(15, 15, 10);
		System.out.println(s.toString());
		s.addObstacle(new Objekt(30, 15, -10, 200, "OBSTAC1"));
		System.out.println(s.toString());
		
		System.out.println("======= test vektorov");
		Field f = new Field(0, 0);
		f.setAngle(Math.asin(3/5.0));
		System.out.println(Math.toDegrees(Math.asin(3/5.0)));
		f.setForce(5);
		System.out.println(f.getVector());*/
		System.out.println(getVector(1.4142135624, Math.toRadians(90)));
		
	}

}
